import Keycloak from 'keycloak-js';
import { useEffect, useRef, useState } from 'react';
import { useAppDispatch, useAuthorizationCache } from '../store';

import { authorized } from '../store/authSlice';

export type AuthenticationHeader = {
  username: string;
  sessionID: string;
  jwt: string;
};

const domain = import.meta.env.BACKEND_URL;
const useruri = import.meta.env.BACKEND_USER;

export const fetchSettings: RequestInit = {
  method: 'GET',
  credentials: 'include',
  redirect: 'follow',
};

export const useAuth = () => {
  const dispatch = useAppDispatch();
  const auth = useAuthorizationCache();

  const handleError = (err: any) => {
    if (import.meta.env.SKIP_LOGIN) {
      console.warn('skipping login');
      dispatch(
        authorized({
          username: 'UserID',
          sessionID: 'SessionID',
          jwt: 'jwt',
          authorized: true,
        })
      );
      return;
    } else {
      console.error(err);
    }
  };

  const login = () => {
    fetch(`${domain}${useruri}/headers`, fetchSettings)
      .then((res) =>
        res
          .json()
          .then((res: AuthenticationHeader) => {
            dispatch(
              authorized({
                username: res.username,
                sessionID: res.sessionID,
                jwt: res.jwt,
                authorized: true,
              })
            );
          })
          .catch(handleError)
      )
      .catch(handleError);
  };

  const fetchAuthenticated = (input: RequestInfo | URL, init?: RequestInit | undefined): Promise<Response> => {
    if (!init) init = fetchSettings;

    init.credentials = 'include';
    init.redirect = 'follow';
    init.method = init.method || 'GET';
    init.headers = {
      'Content-Type': 'application/json',
      sessionid: auth.sessionID || '',
      //   Authorization: `Bearer ${auth.jwt}`,
      ...init.headers,
    };

    return fetch(input, init);
  };

  return { login, fetchAuthenticated };
};

// export useAuth;
